
#include <cstdio>
#include <cstring>
#include <algorithm>
#include <map>
#include <queue>

using namespace std;


const int maxn = 12;
const int maxk = 12;
const int max_sq_comb = 1 << maxk;
const int maxf = maxn * maxn * maxn * maxn * max_sq_comb;

const int pos_winning = 1;
const int pos_losing = 2;

int mv[4][2] = {{-1, 0}, {1, 0}, {0, -1}, {0, 1}};


int n, m;
int board[maxn + 2][maxn + 2];

int p;
int sq_count;
int sq_comb;
int key[maxn + 2][maxn + 2];
map< int, pair<int, int> > num_to_sq;
map< pair<int, int>, int > sq_to_num;

int p1x, p1y, p2x, p2y;
int dx, dy;

char f[maxf];
char nei_cnt[maxf];
queue<int> q;


void read_test()
{
    scanf("%d%d", &n, &m);

    for (int i = 0; i <= n + 1; i++)
    {
        board[i][0] = board[i][m + 1] = 1;
    }

    for (int i = 1; i <= m; i++)
    {
        board[0][i] = board[n + 1][i] = 1;
    }

    for (int i = 1; i <= n; i++)
    {
        for (int j = 1; j <= m; j++)
        {
            scanf("%d", board[i] + j);
        }
    }

    sq_count = 0;
    num_to_sq.clear();
    sq_to_num.clear();
    memset(key, -1, sizeof(key));

    scanf("%d", &p);
    for (int i = 0; i < p; i++)
    {
        int x, y, a, b;
        scanf("%d%d%d%d", &x, &y, &a, &b);

        pair<int, int> sq = make_pair(a, b);
        if (sq_to_num.find(sq) == sq_to_num.end())
        {
            sq_to_num[sq] = sq_count;
            num_to_sq[sq_count] = sq;
            sq_count++;
        }

        int num = sq_to_num[sq];
        key[x][y] = num;
    }

    sq_comb = (1 << sq_count);

    scanf("%d%d", &p1x, &p1y);
    scanf("%d%d", &p2x, &p2y);
    scanf("%d%d", &dx, &dy);
}


bool is_free(int x, int y, int c)
{
    if (x > 0 && y > 0 && x <= n && y <= m)
    {
        pair<int, int> sq = make_pair(x, y);
        if (sq_to_num.find(sq) == sq_to_num.end())
        {
            return board[x][y] == 0;
        }
        else
        {
            int num = sq_to_num[sq];
            if (c & (1 << num))
            {
                return !board[x][y] == 0;
            }
            else
            {
                return board[x][y] == 0;
            }
        }
    }
    else
    {
        return false;
    }
}


int make_fi(int x1, int y1, int x2, int y2, int c)
{
    x1--;
    y1--;
    x2--;
    y2--;

    return ((((x1 * m + y1) * n + x2) * m + y2) * sq_comb + c);
}


void decode_fi(int fi, int &x1, int &y1, int &x2, int &y2, int &c)
{
    c = fi % sq_comb;
    fi /= sq_comb;

    y2 = fi % m + 1;
    fi /= m;

    x2 = fi % n + 1;
    fi /= n;

    y1 = fi % m + 1;
    fi /= m;

    x1 = fi + 1;
}


void count_nei()
{
    for (int c = 0; c < sq_comb; c++)
    {
        for (int x1 = 1; x1 <= n; x1++)
        {
            for (int y1 = 1; y1 <= m; y1++)
            {
                if (is_free(x1, y1, c))
                {
                    for (int x2 = 1; x2 <= n; x2++)
                    {
                        for (int y2 = 1; y2 <= m; y2++)
                        {
                            if (is_free(x2, y2, c))
                            {
                                int fi = make_fi(x1, y1, x2, y2, c);

                                nei_cnt[fi] = 1;

                                if (key[x1][y1] != -1)
                                {
                                    pair<int, int> sq = num_to_sq[key[x1][y1]];
                                    int sq_x = sq.first;
                                    int sq_y = sq.second;

                                    if (x2 != sq_x || y2 != sq_y)
                                    {
                                        nei_cnt[fi]++;
                                    }
                                }

                                for (int i = 0; i < 4; i++)
                                {
                                    int x = x1 + mv[i][0];
                                    int y = y1 + mv[i][1];

                                    if (is_free(x, y, c))
                                    {
                                        nei_cnt[fi]++;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}


void change_state(int src_fi, int dest_fi)
{
    if (f[src_fi] == 0)
    {
        if (f[dest_fi] == pos_losing)
        {
            f[src_fi] = pos_winning;
            q.push(src_fi);
        }
        else
        {
            nei_cnt[src_fi]--;
            if (nei_cnt[src_fi] == 0)
            {
                f[src_fi] = pos_losing;
                q.push(src_fi);
            }
        }
    }
}


void insert_terminals()
{
    for (int c = 0; c < sq_comb; c++)
    {
        if (is_free(dx, dy, c))
        {
            for (int x = 1; x <= n; x++)
            {
                for (int y = 1; y <= m; y++)
                {
                    if (is_free(x, y, c))
                    {
                        int fi = make_fi(dx, dy, x, y, c);
                        f[fi] = pos_winning;
                        q.push(fi);

                        if (x != dx || y != dy)
                        {
                            fi = make_fi(x, y, dx, dy, c);
                            f[fi] = pos_losing;
                            q.push(fi);
                        }
                    }
                }
            }
        }
    }
}


void compute_positions()
{
    while (q.size() > 0)
    {
        int fi = q.front();
        q.pop();

        int x1, y1, x2, y2, c;
        decode_fi(fi, x1, y1, x2, y2, c);

        // if the start position is determined
        if (x1 == p1x && y1 == p1y && x2 == p2x && y2 == p2y && c == 0)
        {
            break;
        }

        // the other player has not done anything
        int src_fi = make_fi(x2, y2, x1, y1, c);
        change_state(src_fi, fi);

        // the other player has switched a key
        if (key[x2][y2] != -1)
        {
            int num = key[x2][y2];
            pair<int, int> sq = num_to_sq[num];
            if (sq.first != x1 || sq.second != y1)
            {
                int nc = c ^ (1 << num);
                int src_fi = make_fi(x2, y2, x1, y1, nc);
                change_state(src_fi, fi);
            }
        }

        // the other player has moved
        for (int i = 0; i < 4; i++)
        {
            int nx = x2 + mv[i][0];
            int ny = y2 + mv[i][1];

            if (is_free(nx, ny, c))
            {
                int src_fi = make_fi(nx, ny, x1, y1, c);
                change_state(src_fi, fi);
            }
        }
    }
}


void solve_test()
{
    memset(f, 0, sizeof(f));
//    q.c.clear();

    insert_terminals();
    count_nei();
    compute_positions();
}


void write_test()
{
    printf("%d\n", f[make_fi(p1x, p1y, p2x, p2y, 0)]);
}


int main()
{
    read_test();
    solve_test();
    write_test();

    return 0;
}
